"""Este módulo contém funções que geram constantes dinâmicas a partir da API"""

from typing import Any, Literal
import httpx
from os import getenv

from siga_mcp._types import EquipeGeralType, SituacaoUsuarioType
from siga_mcp.clients import langfuse
from siga_mcp.constants import (
    MATRICULA_USUARIO_ATUAL,
    MCP_TRANSPORT,
    NOME_USUARIO_ATUAL,
)
from siga_mcp.prompt import Prompt
from siga_mcp.utils import converter_data_siga, dict_to_markdown
from siga_mcp.xml_builder import XMLBuilder


# Esta função é chamada pela MCP tool e gera os dados reais para a CONSTANTE
# Busca usuários de uma equipe filtrados por gerente responsável, descrição da equipe e situação do usuário
# Contém toda a lógica de negócio: validações, normalização, requisições HTTP e processamento
# Função SÍNCRONA que faz o trabalho pesado e retorna XML com os resultados
def listar_usuarios_equipe_por_gerente(
    matricula_gerente: str | int | Literal["CURRENT_USER"] | None = None,
    descricao_equipe: EquipeGeralType | None = None,
    situacao_usuario: SituacaoUsuarioType | None = None,
) -> str:
    import requests

    # VALIDAÇÃO E NORMALIZAÇÃO DA EQUIPE (se fornecida)
    equipe_final = ""
    if descricao_equipe is not None:
        # Busca a equipe correta na constante EQUIPE_GERAL_TO_NUMBER ignorando maiúsculas/minúsculas
        equipe_normalizada = next(
            (
                key
                for key in EQUIPE_GERAL_TO_NUMBER.keys()
                if str(key).lower() == str(descricao_equipe).lower()
            ),
            None,
        )

        if equipe_normalizada is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "tipo_erro": "equipe_invalida",
                        "equipe_informada": descricao_equipe,
                        "mensagem": f"Equipe '{descricao_equipe}' não encontrada na constante EQUIPE_GERAL_TO_NUMBER",
                        "equipes_validas": list(EQUIPE_GERAL_TO_NUMBER.keys()),
                    }
                ],
                root_element_name="erro_validacao",
                item_element_name="erro",
                root_attributes={
                    "sistema": "SIGA",
                    "funcao": "listar_usuarios_equipe_por_gerente",
                },
                custom_attributes={"sistema": "SIGA"},
            )

        # NORMALIZA: converte nome → código
        equipe_final = EQUIPE_GERAL_TO_NUMBER[equipe_normalizada]

    # VALIDAÇÃO E NORMALIZAÇÃO DA SITUAÇÃO (se fornecida)
    situacao_final = ""
    if situacao_usuario is not None:
        # Busca a situação correta na constante SITUACAO_USUARIO_TO_NUMBER ignorando maiúsculas/minúsculas
        situacao_normalizada = next(
            (
                key
                for key in SITUACAO_USUARIO_TO_NUMBER.keys()
                if str(key).lower() == str(situacao_usuario).lower()
            ),
            None,
        )

        if situacao_normalizada is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "tipo_erro": "situacao_invalida",
                        "situacao_informada": situacao_usuario,
                        "mensagem": f"Situação '{situacao_usuario}' não encontrada na constante SITUACAO_USUARIO_TO_NUMBER",
                        "situacoes_validas": list(SITUACAO_USUARIO_TO_NUMBER.keys()),
                    }
                ],
                root_element_name="erro_validacao",
                item_element_name="erro",
                root_attributes={
                    "sistema": "SIGA",
                    "funcao": "listar_usuarios_equipe_por_gerente",
                },
                custom_attributes={"sistema": "SIGA"},
            )

        # NORMALIZA: converte nome → número
        situacao_final = SITUACAO_USUARIO_TO_NUMBER[situacao_normalizada]

    try:
        response = requests.post(
            "https://ava3.uniube.br/ava/api/usuarios/listarUsuariosEquipePorGerente/",
            json={
                "apiKey": getenv("AVA_API_KEY"),
                "matriculaGerente": str(matricula_gerente) if matricula_gerente else "",
                "equipe": str(equipe_final) if equipe_final else "",
                "situacaoUsuario": str(situacao_final) if situacao_final else "",
            },
        )

        json_response = response.json()

        # VERIFICAR SE JSON_RESPONSE É None
        if json_response is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "mensagem": "API retornou resposta vazia. Verifique a configuração da API key.",
                    }
                ],
                root_element_name="resultado",
                item_element_name="item",
                custom_attributes={"sistema": "SIGA"},
            )

        result_data = json_response.get("result")

        # Trata a resposta
        if result_data is None or len(result_data) == 0:
            data_final = [
                {
                    "status": "aviso",
                    "mensagem": "Nenhum usuário encontrado para os filtros informados. Verifique se você é gerente de uma equipe ou ajuste os filtros de busca.",
                }
            ]

            # Retorna XML de aviso em vez dos dados
            return XMLBuilder().build_xml(
                data=data_final,
                root_element_name="usuarios_equipe_gerente",
                item_element_name="resultado",
                root_attributes={
                    "matriculaGerente": str(matricula_gerente)
                    if matricula_gerente
                    else "",
                    "equipe": str(equipe_final) if equipe_final else "",
                    "situacaoUsuario": str(situacao_final) if situacao_final else "",
                },
                custom_attributes={"sistema": "SIGA"},
            )
        else:
            # Se há resultados, processar normalmente
            return XMLBuilder().build_xml(
                data=result_data,
                root_element_name="usuarios_equipe_gerente",
                item_element_name="usuario_equipe_gerente",
                root_attributes={
                    "matriculaGerente": str(matricula_gerente)
                    if matricula_gerente
                    else "",
                    "equipe": str(equipe_final) if equipe_final else "",
                    "situacaoUsuario": str(situacao_final) if situacao_final else "",
                },
                custom_attributes={"sistema": "SIGA"},
            )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


# Constante com a lista de colaboradores de todas as equipes da empresa, para ser usado no prompt do sistema, durante o uso do chat do SIGA.
COLABORADORES_PROMPT: str = listar_usuarios_equipe_por_gerente()


# Obter usuário responsável para criação de OS Sistemas e Infraestrutura.
# Usado para montar o Docstring e validação na função, caso o usuário informa matrícula que não está na lista
def obter_usuarios_responsavel(area: int) -> tuple[str, set[str], list[str]]:
    # Determinar nome da área para mensagens de erro
    nome_area = "Sistemas" if area == 1 else "Infraestrutura"

    try:
        # Fazer requisição HTTP para buscar usuários responsáveis
        with httpx.Client(timeout=60.0) as client:
            response = client.post(
                "https://ava3.uniube.br/ava/api/usuarios/buscarUsuarioResponsavelOsSigaIA/",
                json={
                    "apiKey": getenv("AVA_API_KEY"),
                    "area": area,
                },
            )

            # Extrair dados JSON da resposta
            json_data = response.json()
            data: list[dict[str, Any]] | dict[str, Any] = (  # type: ignore
                json_data
                if isinstance(json_data, list)
                else json_data.get("result", [])
            )

            # Validar se recebeu dados válidos
            if not data or not isinstance(data, list):
                return (
                    f"        - Erro ao carregar usuários responsáveis de {nome_area}",
                    set(),
                    [],
                )

            # Remover duplicatas usando dict (chave = ID do usuário)
            usuarios_unicos: dict[str, dict[str, Any]] = {}
            for usuario in data:
                if (
                    isinstance(usuario, dict)  # type: ignore
                    and "USUARIO" in usuario
                    and "NOME" in usuario
                ):
                    usuarios_unicos[usuario["USUARIO"]] = usuario

            # Verificar se encontrou usuários válidos
            if not usuarios_unicos:
                return (
                    f"        - Nenhum usuário responsável encontrado para {nome_area}",
                    set(),
                    [],
                )

            # 📝 GERAR LISTA FORMATADA PARA DOCSTRING (ordenada alfabeticamente)
            usuarios_ordenados = sorted(
                usuarios_unicos.values(), key=lambda x: x["NOME"]
            )
            docstring = "\n".join(
                [
                    f'        - "{usuario["NOME"]}" (ID: {usuario["USUARIO"]})'
                    for usuario in usuarios_ordenados
                ]
            )

            # 🔍 GERAR SET DE IDS PARA VALIDAÇÃO RÁPIDA SE O USUÁRIO ESTÁ NA LISTA DE RESPONSÁVEIS DA ÁREA
            ids_validacao = {
                str(usuario["USUARIO"]) for usuario in usuarios_unicos.values()
            }

            # 🔍 GERAR LISTA PARA MENSAGENS DE ERRO, PARA AVISAR QUE O USUÁRIO NÃO ESTÁ NA LISTA DE RESPONSÁVEIS DAQUELA ÁREA
            usuarios_para_erro = [
                f'"{usuario["NOME"]}" (ID: {usuario["USUARIO"]})'
                for usuario in usuarios_ordenados
            ]

            # Retorna 3 resultados, para docstring, set de ids para validação rápida e lista para mensagens de erro
            return (docstring, ids_validacao, usuarios_para_erro)

    except Exception:
        # Retornar erro em caso de falha na requisição ou processamento
        return (
            f"        - Erro ao carregar usuários responsáveis de {nome_area}",
            set(),
            [],
        )


# ✅ CONSTANTES CACHED - Executam uma vez quando o módulo é carregado
# 📊 Buscar dados para Sistemas (área 1)
USUARIOS_SISTEMAS_DOCSTRING, USUARIOS_SISTEMAS_IDS, USUARIOS_SISTEMAS_PARA_ERRO = (
    obter_usuarios_responsavel(1)
)
# 🔧 Buscar dados para Infraestrutura (área 2)
(
    USUARIOS_INFRAESTRUTURA_DOCSTRING,
    USUARIOS_INFRAESTRUTURA_IDS,
    USUARIOS_INFRAESTRUTURA_PARA_ERRO,
) = obter_usuarios_responsavel(2)

# Lista dos Tipos de Atendimento.
TYPE_TO_NUMBER = {
    "Suporte Sistema": 1,
    "Implementação": 2,
    "Manutenção Corretiva": 3,
    "Reunião": 4,
    "Treinamento": 5,
    "Mudança de Escopo": 20,
    "Anexo": 12,
    "Suporte Infraestrutura": 13,
    "Monitoramento": 21,
    "Incidente": 23,
    "Requisição": 24,
}


# Listagem para a Combo Tipos.
TIPO_TO_NUMBER_ATENDIMENTO_AVULSO = {
    "Suporte Sistema": 1,
    "Manutenção de Banco": 10,
    "Atividade Interna": 19,
}

# Listagem para a Combo Tipos para atendimento avulso de infraestrutura.
TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA = {
    "Atividade Interna": 19,
    "Implementação": 15,
    "Incidente": 23,
    "Manutenção Corretiva": 17,
    "Manutenção de Banco": 10,
    "Manutenção Preventiva": 16,
    "Monitoramento": 22,
    "Requisição": 24,
    "Suporte": 14,
    "Treinamento": 18,
}

# Listagem para a Combo Tipos para OS.
TIPO_TO_NUMBER_OS_SISTEMAS = {
    "Implementação": 2,
    "Manutenção Corretiva": 3,
    "Monitoramento": 21,
    "Mudança de Escopo": 20,
    "Suporte Infraestrutura": 13,
    "Suporte Sistema": 1,
    "Treinamento": 5,
}

# Listagem para a Combo Origem.
ORIGEM_TO_NUMBER = {
    "E-mail": 1,
    "Pessoalmente": 2,
    "Teams": 3,
    "Telefone": 4,
    "WhatsApp": 5,
    "Plantão": 7,
    "SAE": 11,
}

# Listagem para a Combo Origem para OS.
ORIGEM_OS_TO_NUMBER = {
    "E-mail": 1,
    "Pessoalmente": 2,
    "Teams": 3,
    "Telefone": 4,
    "WhatsApp": 5,
    "Plantão": 7,
    "SAE": 11,
    "SATIC": 8,
    "Siga": 6,
}


# Listagem para a Combo Sistema.
SISTEMA_TO_NUMBER = {
    "Abaris": 285,
    "Administrar Permissões e Acesso - Segurança": 45,
    "Analytics / BI": 282,
    "Analytics / BI (Administrativos/Hospitalares)": 305,
    "APP Pega Plantão": 304,
    "Assinatura Digital / Bird ID": 286,
    "Controle de Contratos": 106,
    "Custo/Orçamento Institucional": 226,
    "GEM - Aplicativo de Apoio": 92,
    "Intranet": 302,
    "MV - Almoxarifado": 154,
    "MV - Ambulatório": 155,
    "MV - Apoio à TI": 156,
    "MV - Auditoria e Glosa": 157,
    "MV - Caixa": 158,
    "MV - CCIH": 159,
    "MV - Central de Marcação": 160,
    "MV - Centro Cirúrgico e Obstétrico": 161,
    "MV - CME": 303,
    "MV - Conciliação de Convênios": 163,
    "MV - Contabilidade": 164,
    "MV - Contas a Pagar": 165,
    "MV - Contas a Receber": 166,
    "MV - Controle Bancário": 167,
    "MV - Custos": 217,
    "MV - Diagnóstico por Imagem": 168,
    "MV - Diretoria Clínica": 169,
    "MV - Faturamento de Convênios e Particulares": 170,
    "MV - Faturamento SUS": 171,
    "MV - Gerenciamento de Projetos": 239,
    "MV - Gestão de Documentos": 238,
    "MV - Gestão de Ocorrências": 236,
    "MV - Gestão de Riscos": 237,
    "MV - Higienização": 172,
    "MV - HMed": 264,
    "MV - Internação": 173,
    "MV - Laboratório de Análises Clínicas": 174,
    "MV - Lavanderia e Rouparia": 175,
    "MV - Manutenção": 176,
    "MV - MovDoc": 177,
    "MV - Nutrição": 178,
    "MV - Patrimônio": 179,
    "MV - PEP": 140,
    "MV - Repasse Médico": 219,
    "MV - SAC": 180,
    "MV - SAME": 181,
    "MV - Sistema de Apoio": 129,
    "MV - Tesouraria": 182,
    "MV - Urgência": 223,
    "Prefeitura Universitária": 124,
    "PROT - Protocolo": 108,
    "RCI  - Avaliador (Cópias e Impressões)": 4,
    "RH - Controle de Convênios": 121,
    "RH - Plano de Cargos e Salários": 115,
    "RH - Sistema de Apoio ao Recursos Humanos": 107,
    "RMS - Almoxarifado": 1,
    "RMS - Aprovador": 6,
    "RMS - Avaliador": 8,
    "RMS - Compras": 2,
    "RMS - Gestão de Logística": 122,
    "RMS - Gestão de Serviços": 103,
    "RMS - Gestão de Transporte": 113,
    "RMS - Marketing & Comunicacao": 138,
    "RMS - Patrimônio": 232,
    "RMS - Requisitante": 10,
    "RPA - Recibo de Pagamento Autônomo (Pessoa Física)": 274,
    "Sapiens - Contabilidade": 250,
    "Sapiens - Contas a Pagar": 266,
    "Sapiens - Contas a Receber": 269,
    "Sapiens - Fluxo de Caixa": 268,
    "Sapiens - Recebimento": 267,
    "Sapiens - Sistema de Apoio": 259,
    "Sapiens - Tesouraria": 270,
    "Sapiens - Tributos": 249,
    "Senior - Administração de Pessoal": 184,
    "Senior - Controle de Acesso": 185,
    "Senior - Controle de Ponto": 183,
    "Senior - Jurídico Trabalhista": 278,
    "Senior - Medicina e Segurança do Trabalho": 186,
    "SGA - Acadêmico": 131,
    "SGA - Atividades Administrativas": 110,
    "SGA - Carteirinhas": 125,
    "SGA - Censo": 290,
    "SGA - Contabilidade": 119,
    "SGA - Controle de Ponto": 30,
    "SGA - Controle de Reuniões do Conselho e Câmara": 116,
    "SGA - CPA": 287,
    "SGA - Estágio": 24,
    "SGA - Estágio (Novo)": 224,
    "SGA - Extrator de Dados": 227,
    "SGA - Financeiro": 38,
    "SGA - Formandos": 19,
    "SGA - FORMANDOS (DIPLOMA DIGITAL) - ANTIGO": 277,
    "SGA - FORMANDOS (DIPLOMA DIGITAL) - ATUAL": 300,
    "SGA - Pesquisa": 109,
    "SGA - PIME": 120,
    "SGA - Planejamento EAD": 127,
    "SGA - Pós-Graduação e Extensão": 23,
    "SGA - Pós-Graduação .Net": 248,
    "SGA - Processos Seletivos": 118,
    "SGA - Produção de Materiais Didáticos": 49,
    "SGA - Roteiros": 128,
    "SGA - SISCAP": 272,
    "SGA - Telemarketing": 37,
    "SGA - WEB Administrativo": 222,
    "SGB - Biblioteca": 126,
    "SGS - Clínicas Integradas": 104,
    "SGS - Laboratorio Protese": 230,
    "SGV - Administrativo": 51,
    "SGV - Ambulatório": 52,
    "SGV - Cirúrgico": 53,
    "SGV - Farmácia": 54,
    "SGV - Financeiro": 55,
    "SGV - Financeiro .Net": 229,
    "SGV - Imagem": 221,
    "SGV - Internação": 56,
    "SGV - Laboratório": 57,
    "SGV - LMVP": 58,
    "SGV - Patologia": 59,
    "SGV - Recepção": 60,
    "SIGA - Gestão de Solicitações a DTD / Atividades": 143,
    "Sistemas AVA": 271,
    "Site Institucional": 275,
    "Site UAGRO - Dottatec": 284,
    "Site Universidade do Agro (Drupal)": 301,
    "SITES SAUDE / HOSPITAIS": 295,
    "Sophia": 262,
    "Uniube Sistemas Integrados - USI": 291,
    "Uniube.br - Acesso restrito": 279,
    "Consist Gem - Contabilidade (INATIVO)": 144,
    "Consist Gem - Contas a Pagar (INATIVO)": 146,
    "ORSE - Aplicativo de Apoio (INATIVO)": 93,
    "SGA - Digitalizações (INATIVO)": 43,
    "SGA - Pesquisa MPHU (INATIVO)": 133,
}


# Listagem para a Combo Categoria da Infraestrutura.
CATEGORIA_TO_NUMBER = {
    "AD - Alterar Configuração do Domínio": 13,
    "AD - Criar usuário": 93,
    "AD - Desbloquear usuário": 68,
    "AD - Excluir/Bloquear usuário": 67,
    "AD - Liberar permissões de acesso": 11,
    "AD - Redefinir Senha": 12,
    "AD - Suporte/Dúvidas/Outros": 39,
    "BD - Alterar tabela": 72,
    "BD - Atividade Interna": 94,
    "BD - Atualizar esquema": 56,
    "BD - Corrigir lentidão e bloqueios": 57,
    "BD - Criar tabela/índice": 71,
    "BD - Criar usuário": 54,
    "BD - Liberar acessos/permissões": 69,
    "BD - Monitorar rotina de backups e testes de restauração": 55,
    "BD - Reiniciar tabela / Tablespace": 53,
    "BD - Restauração de LOG": 70,
    "BD - Tunning de instrução": 58,
    "DB - Suporte/Dúvidas/Outros": 96,
    "DPO - Analisar contratos": 74,
    "DPO - Analisar/Autorizar autorização de dados e imagens": 75,
    "DPO - Conscientizar sobre segurança digital": 76,
    "DPO - Criar/Implementar política de segurança": 77,
    "E-mail - Alterar Colaborador Responsável": 5,
    "E-mail - Configurar Google Workspace": 9,
    "E-mail - Configurar primeiro acesso": 8,
    "E-mail - Criar e-mail": 6,
    "E-mail - Desbloquear e-mail": 78,
    "E-mail - Excluir/Bloquear e-mail": 79,
    "E-mail - Redefinir senha": 7,
    "E-mail - Suporte/Dúvidas/Outros": 40,
    "Hardware - Atualizar driver(s)/Firmware(s)/Limpeza computador/notebook": 35,
    "Hardware - Atualizar driver(s)/Firmware(s)/Limpeza impressora/scanner": 65,
    "Hardware - Backup": 24,
    "Hardware - Consertar computador/notebook": 73,
    "Hardware - Consertar/Trocar impressora/scanner": 80,
    "Hardware - Formatar": 25,
    "Hardware - Instalar Antivírus": 34,
    "Hardware - Instalar/Desinstalar/Atualizar Software": 26,
    "Hardware - Suporte/Dúvidas/Outros": 27,
    "Inclusão / Remoção de Colaboradores": 62,
    "Liberar dispositivo de armazenamento": 97,
    "Publicação - AVA": 66,
    "Rede - Alterar perfil de acesso": 2,
    "Rede - Ativar/Crimpar Ponto de Rede": 19,
    "Rede - Configurar Firewall": 4,
    "Rede - Criar/Alterar regra Firewall": 3,
    "Rede - Instalar/Configurar/Atualizar AP/Câmera/Router/Voip": 22,
    "Rede - Instalar/Configurar/Atualizar controle de acesso/catraca": 23,
    "Rede - Instalar/Configurar/Atualizar REP": 21,
    "Rede - Instalar/Configurar/Atualizar Switch/VLAN": 20,
    "Rede - Liberar internet": 81,
    "Rede - Suporte VPN": 60,
    "Rede - Suporte/Dúvidas/Outros": 41,
    "Segurança - Investigar ataques cibernéticos": 83,
    "Segurança - Remover ameaças detectadas": 82,
    "Serviços - Atividade interna": 28,
    "Serviços - Empréstimo de Equipamento": 42,
    "Serviços - Realizar auditoria/Criar relatório": 1,
    "Serviços - Transferir/Recolher equipamento": 36,
    "Serviços - Treinamento": 29,
    "Servidores - Alterar configuração": 15,
    "Servidores - Atualizar driver(s)/Firmware(s)/Limpeza": 89,
    "Servidores - Atualizar/Reiniciar": 16,
    "Servidores - Criar usuário": 85,
    "Servidores - Disparar/Conferir/Restaurar backup": 18,
    "Servidores - Excluir/Bloquear Usuário": 84,
    "Servidores - Liberar/Bloquear permissões": 86,
    "Servidores - Manutenção Corretiva": 88,
    "Servidores - Manutenção Preventiva": 87,
    "Sistemas - Desbloquear usuário": 49,
    "Sistemas - Instalar sistema": 50,
    "Sistemas - Liberar Permissões": 91,
    "Sistemas - Redefinir senha": 51,
    "Sistemas - Retirar Permissões": 90,
    "Sistemas - Suporte/Dúvidas/Outros": 52,
    "Telefonia - Atualizar aparelho": 92,
    "Telefonia - Configurar aparelho": 44,
    "Telefonia - Consertar/Trocar aparelho": 45,
    "Telefonia - Suporte/Dúvidas/Outros": 46,
    "Verificar log de eventos": 98,
    "AD - Atribuir Direitos de Acesso em Pasta/Impressora (INATIVO)": 32,
    "AD - Criar/Renomear/Bloquear/Desbloquear usuário (INATIVO)": 10,
    "Alterar REP (INATIVO)": 63,
    "Catracas - Manutenção Corretiva/Preventiva (INATIVO)": 47,
    "Coletor Biométrico - Manutenção Corretiva/Preventiva (INATIVO)": 48,
    "DPO (INATIVO)": 64,
    "Equipamentos - Instalar/Desinstalar (INATIVO)": 30,
    "Equipamentos - Manutenção Corretiva/Preventiva (INATIVO)": 37,
    "Equipamentos - Suporte/Dúvida/Outros (INATIVO)": 31,
    "Firewall - Suporte/Dúvida/Outros (INATIVO)": 61,
    "Internet - Suporte/Dúvidas/Outros (INATIVO)": 43,
    "Servidores - Criar/Configurar (INATIVO)": 17,
    "Servidores - Criar/Deletar Usuários e/ou Diretórios (INATIVO)": 33,
    "Servidores - Manutenção Preventiva/Corretiva (INATIVO)": 14,
    "Sistemas - Liberar/Retirar Permissão (INATIVO)": 59,
}

# Listagem para a Combo Equipe.
EQUIPE_TO_NUMBER = {
    "SGA - Acadêmico": "ACAD",
    "RMS (Requisições, Materiais e Serviços)": "RMS",
    "SGA - Financeiro": "FIN",
    "Recursos Humanos": "RH",
    "Financeiro e Contábil": "FINCONT",
    "Saúde": "SAUDE",
    "SGA - Web": "SGAWEB",
    "Administador de Banco de Dados": "DBA",
    "Escritório de Projetos": "PROJ",
    "Analytics": "Analytics",
    "Equipe AVA": "AVA",
}

# Listagem para a Combo Equipe Infraestrutura.
EQUIPE_INFRAESTRUTURA_TO_NUMBER = {
    "Administador de Banco de Dados": "DBA",
    "Gerenciamento de Redes": "REDES",
    "Gerenciamento de Redes - Linux": "LINUX",
    "Gerenciamento de Redes - Windows": "WINDOWS",
    "Help-Desk - Aeroporto": "Help Aero",
    "Help-Desk - Ambulatório": "Help Amb",
    "Help-Desk - Araxá": "Help Ara",
    "Help-Desk - Centro": "Help Cen",
    "Help-Desk - HR": "Help HR",
    "Help-Desk - HVU": "Help HVU",
    "Help-Desk - IMM": "Help IMM",
    "Help-Desk - MPHU": "Help MPHU",
    "Help-Desk - NPG": "Help NPG",
    "Help-Desk - UPA Mirante": "Help UPA_M",
    "Help-Desk - UPA São Benedito": "Help UPASB",
    "Help-Desk - Via Centro": "Help Mar",
    "Help-Desk - Vila Gávea": "Help Vila",
    "LIAE - Aeroporto": "LIAE Aero",
    "LIAE - Via Centro": "LIAE Mar",
    "Ouvidoria / Telefonia": "OUVIDORIA",
    "Proteção de dados": "DPO",
    "Publicação AVA": "Pub-AVA",
}

EQUIPE_GERAL_TO_NUMBER = {
    "SGA - Acadêmico": "ACAD",
    "RMS (Requisições, Materiais e Serviços)": "RMS",
    "SGA - Financeiro": "FIN",
    "Recursos Humanos": "RH",
    "Financeiro e Contábil": "FINCONT",
    "Saúde": "SAUDE",
    "SGA - Web": "SGAWEB",
    "Administador de Banco de Dados": "DBA",
    "Escritório de Projetos": "PROJ",
    "Analytics": "Analytics",
    "Equipe AVA": "AVA",
    "Gerenciamento de Redes": "REDES",
    "Gerenciamento de Redes - Linux": "LINUX",
    "Gerenciamento de Redes - Windows": "WINDOWS",
    "Help-Desk - Aeroporto": "Help Aero",
    "Help-Desk - Ambulatório": "Help Amb",
    "Help-Desk - Araxá": "Help Ara",
    "Help-Desk - Centro": "Help Cen",
    "Help-Desk - HR": "Help HR",
    "Help-Desk - HVU": "Help HVU",
    "Help-Desk - IMM": "Help IMM",
    "Help-Desk - MPHU": "Help MPHU",
    "Help-Desk - NPG": "Help NPG",
    "Help-Desk - UPA Mirante": "Help UPA_M",
    "Help-Desk - UPA São Benedito": "Help UPASB",
    "Help-Desk - Via Centro": "Help Mar",
    "Help-Desk - Vila Gávea": "Help Vila",
    "LIAE - Aeroporto": "LIAE Aero",
    "LIAE - Via Centro": "LIAE Mar",
    "Ouvidoria / Telefonia": "OUVIDORIA",
    "Proteção de dados": "DPO",
    "Publicação AVA": "Pub-AVA",
}

# Listagem para a Combo Projeto.
PROJETO_TO_NUMBER = {
    "Adequações para ONA 2022": 107,
    "Adequações para ONA 2024": 155,
    "Adequações para ONA 2025": 198,
    "Aditivos ao contrato dos alunos": 125,
    "Anonimização de prontuário do paciente": 143,
    "Análise Inicial - Implantação do sistema de imagens na Policlínica": 15,
    "APP do Paciente": 136,
    "Autoria": 77,
    "AVA - CORPORATIVO": 129,
    "Campus Villa Gávea - Infraestrutura e Segurança": 133,
    "Cartão Saúde Uniube": 181,
    "Cartão Vital": 189,
    "Consultoria externa Contábil/Financeiro": 170,
    "Consultoria externa HCM": 171,
    "Controle de limpeza de leitos hospitalares": 145,
    "Controle de registros dos médicos": 137,
    "Criar script de mapeamento das impressoras": 54,
    "Desenvolvimento Componentes / Framework": 188,
    "Desenvolvimento Web / Mobile": 186,
    "Estudo de plataformas de CRM e Vendas": 156,
    "Gestão de Sucesso dos Polos": 118,
    "Graduação - ajustes na transferência, 2ª graduação - 2025": 197,
    "Implantação do módulo jurídico": 99,
    "Implantação do sistema No Harm - Farmácia": 142,
    "Implantação Integração MVPEP e ATRIUM": 146,
    "Implantação UPAs": 131,
    "Integração da modalidade ECG com o PACs": 153,
    "Integração entre Sistema Epimed Monitor UTI e o MVPEP": 174,
    "Integração SAE e Protocolos": 75,
    "ITVix - SIG Polos Integração": 130,
    "Mapeamento AS IS Logística e Central de Malotes": 122,
    "Melhorias e automação de atendimento - MPHU e TakeBlip": 120,
    "Melhorias na Transferência externa e aproveitamento de estudos": 116,
    "Melhorias no módulo de treinamento": 196,
    "Melhorias no Sistema de Geração de Provas e Fechamento de Disciplinas do EAD": 103,
    "Melhorias para SADT - MPHU": 124,
    "Migração .Net (Entity + Crystal)": 101,
    "Migração de sistemas Fox Pro": 138,
    "Migração para o PHP 8": 100,
    "Novo CNES das Clínicas Integradas": 140,
    "Novo formato alfanumérico para o Cadastro Nacional da Pessoa Jurídica (CNPJ)": 205,
    "Operacao Publicacao AVA": 119,
    "Operaçao DPO": 114,
    "Operação Acadêmico": 28,
    "Operação Analytics": 151,
    "Operação AVA": 67,
    "Operação Banco de Dados": 207,
    "Operação Biblioteca": 30,
    "Operação Clínicas": 2,
    "Operação Compras": 3,
    "Operação Financeiro/Contabilidade": 4,
    "Operação Gestão de Relacionamento": 72,
    "Operação Help Desk": 61,
    "Operação HMed": 64,
    "Operação HVU": 5,
    "Operação Infraestrutura": 62,
    "Operação Jurídico Trabalhista": 201,
    "Operação LIAE": 63,
    "Operação Medicina do Trabalho": 199,
    "Operação MV": 6,
    "Operação RH": 7,
    "Operação RMS": 8,
    "Operação Saúde - Web": 187,
    "Operação Segurança do Trabalho": 200,
    "Operação SGA - Financeiro/Contabilidade": 29,
    "Operação Site Institucional": 98,
    "Operação TI": 19,
    "Operação WEB Administrativo": 27,
    "Overmind.ia - Automação entre MV e Convênios": 203,
    "Painéis interativos de sistemas de saúde": 135,
    "Projeto - Fluxo de locação de espaços físicos": 123,
    "Projeto APP Marcação de Ponto para Professores": 139,
    "Projeto App Pega Plantão": 202,
    "Projeto AVA 3.0": 73,
    "Projeto Banco de Questões": 84,
    "Projeto BI": 89,
    "Projeto Carrinhos Beira Leito": 173,
    "Projeto Contratos Empresariais - PROED": 20,
    "Projeto Controle de Acessos dos Hospitais": 157,
    "Projeto Cópia de perfil": 108,
    "Projeto de adequação Rede WIFI": 132,
    "Projeto de Automatização de Convênios do MPHU": 127,
    "Projeto de Controle de Vacinas no HVU": 147,
    "Projeto de desenvolvimento IA para Plano Terapeutico": 204,
    "Projeto de integração Comtele": 93,
    "Projeto de integração Intersaberes": 92,
    "Projeto de Melhoria de Agendamento de Serviços de Transportes": 193,
    "Projeto de melhorias nas Clínicas Integradas": 109,
    "Projeto de Melhorias no Controle de Contratos": 74,
    "Projeto de melhorias no faturamento MPHU": 106,
    "Projeto de melhorias no HVU": 105,
    "Projeto de Melhorias nos Setores Jurídicos": 126,
    "Projeto de melhorias SEU Financeiro": 112,
    "Projeto de Solicitação de Contratação": 110,
    "Projeto Digitalização Secretaria do Conselho Universitário": 160,
    "Projeto Diploma Digital": 97,
    "Projeto Documentação de Telas e Sistemas": 154,
    "Projeto DRG Brasil - Hospitais": 144,
    "Projeto Evolução do Sistema RMS-Almoxarifado": 195,
    "Projeto Fluxo de Situação Acadêmica EAD": 38,
    "Projeto Gestão da Permanência Qualificada": 36,
    "Projeto GIT": 83,
    "Projeto GPQ": 70,
    "Projeto Ilhas de Impressão": 128,
    "Projeto IMM - Implantação Multiempresa": 178,
    "Projeto Implantação Clínicas MV": 190,
    "Projeto Implantação Ábaris - secretaria digital e diplomas": 115,
    "Projeto Implantação Ábaris - XML histórico parcial e oficial": 164,
    "Projeto Inscrição e Matrícula dos Cursos de Graduação": 102,
    "Projeto LGPD": 88,
    "Projeto Limpeza dos Sistemas AVA": 79,
    "Projeto MELHORIA SISTEMA de APOIO RH": 113,
    "Projeto Melhorias no controle de acesso - Hospitais - Campus Centro - Estacionamentos": 177,
    "Projeto Migração para .Net SGA - Financeiro": 31,
    "Projeto Migração para o Sistemas Integrados WEB": 35,
    "Projeto Número de Alunos - Graduação": 149,
    "Projeto Número de Alunos - Pós-Graduação": 150,
    "Projeto Operação Formandos": 152,
    "Projeto Reestruturação do Repasse a Parceiros - PROED": 21,
    "Projeto Revisão Orçamento Institucional": 168,
    "Projeto RH - Análise de Danos Causados pelo Empregado": 185,
    "Projeto RH - Avaliação de Desenvolvimento": 172,
    "Projeto RH - Coleta de assinatura digital dos ASOs": 191,
    "Projeto RH - Plano de Cargos e Salários": 158,
    "Projeto SEAD CONSAE - Etapa 1": 16,
    "Projeto SEAD CONSAE - Etapa 2": 34,
    "Projeto Secretaria Digital": 86,
    "Projeto Sistema de Avaliação da EAD": 78,
    "Projeto Sistemas de Saúde - WEB": 175,
    "Projeto SMS": 85,
    "Projeto Unificação do PIAC": 81,
    "Projeto Unificação do PROEST": 80,
    "Projeto UniFlex": 39,
    "Projeto Universidade do Agro": 111,
    "Projeto Ábaris - Currículo dos cursos em XML": 165,
    "Projeto: Migração do sistema de Compras/Bionexo": 192,
    "Projetos ASSCOM": 90,
    "Projetos de BIs do sistema RMS": 141,
    "Projetos Estágio": 94,
    "Projetos PMO": 96,
    "Projetos PROPEPE": 91,
    "Projetos Setor Financeiro": 148,
    "Publicações AVA": 117,
    "Pós EAD 2.0": 87,
    "Reformulação do SITE MPHU": 180,
    "RMGV": 76,
    "Sistemas Parceiros": 82,
    "Site Hospitalares": 179,
    "Situações acadêmicas da Pós-Graduação": 182,
    "Transformação Digital": 104,
    "Transformação digital - Aproveitamento/transferencia/segunda graduação": 161,
    "Transformação Digital - Reabertura de atividades do AVA": 163,
    "Transformação Digital - Reemissão de boletos (Refit 2024)": 162,
    "Universidade do Agro - Novo site": 183,
    "Universidade MV": 159,
    "Upgrade SO Windows Server 2012 - fim do suporte": 134,
    "Vertifical da Saúde Uniube (projeto)": 206,
}


# Listagem para a Combo Tipos para OS.
LINGUAGEM_TO_NUMBER_OS_SISTEMAS = {
    "C#": 1,
    "Fox": 2,
    "SQL": 3,
    "ASP.Net": 4,
    "Access": 5,
    "PHP": 6,
    "Extrator de Dados": 7,
    "MV Painel de Indicadores": 8,
    "MV Editor": 9,
    "Gerador de Relatórios": 10,
    "Gerador de Cubos": 11,
    "Power BI": 18,
    "Gerador de Tela": 12,
    "Editor de Regra": 13,
    "Delphi": 14,
    "Script SO": 15,
    "Node.js": 23,
    "Senior - Gerador de visão dinâmica": 24,
    "Analytics": 20,
}

# Listagem para a Combo Interna para OS.
OS_INTERNA_OS_TO_NUMBER = {
    "Sim": 1,
    "Não": 0,
}


# Listagem para a Combo Status para OS.
STATUS_OS_TO_NUMBER = {
    "Concluída": 8,
    "Concluída por Encaminhamento": 9,
    "Concluída por substituição": 10,
    "Em Atendimento": 3,
    "Em Implantação": 7,
    "Em Teste": 5,
    "Não Planejada": 1,
    "Pendente-Aprovação": 96,
    "Pendente-Atendimento": 2,
    "Pendente-Atualização de Versão": 93,
    "Pendente-AVA": 94,
    "Pendente-Consultoria": 92,
    "Pendente-Equipe Infraestrutura": 95,
    "Pendente-Equipe Manutenção": 88,
    "Pendente-Fornecedor": 97,
    "Pendente-Help-Desk": 87,
    "Pendente-Liberação": 6,
    "Pendente-Marketing": 89,
    "Pendente-Sist. Acadêmicos": 90,
    "Pendente-Sist. Administrativos": 91,
    "Pendente-Teste": 4,
    "Pendente-Usuário": 98,
    "Solicitação em Aprovação": 101,
    "Cancelada-Usuário": 100,
    "Cancelamento DTD | Arquivado": 99,
}


# Listagem para a Combo Criticidade para OS.
CRITICIDADE_OS_TO_NUMBER = {
    "Nenhuma": 0,
    "Baixa": 1,
    "Média": 2,
    "Alta": 3,
}

# Listagem para a Combo Criticidade para OS.
PRIORIDADE_USUARIO_OS_TO_NUMBER = {
    "Nenhuma": 0,
    "Urgente": 1,
    "Alta": 2,
    "Média": 3,
    "Baixa": 4,
}


# Constante para situação do usuário
SITUACAO_USUARIO_TO_NUMBER = {
    "Bloqueado": 0,
    "Ativo": 1,
    "Bloqueado (Afastamento)": 2,
    "Bloqueado pelo RH (Individual)": 3,
    "Bloqueado por Falta de Justificativa de Ponto (Individual)": 4,
    "Bloqueado Licença sem Remuneração": 5,
}

constantes_md = dict_to_markdown(
    ("tipos do atendimento", TYPE_TO_NUMBER),
    ("tipos atendimento avulso sistemas", TIPO_TO_NUMBER_ATENDIMENTO_AVULSO),
    (
        "tipos atendimento avulso infraestrutura",
        TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA,
    ),
    ("tipos atendimentos sistemas os", TIPO_TO_NUMBER_OS_SISTEMAS),
    ("origens", ORIGEM_TO_NUMBER),
    ("origens os", ORIGEM_OS_TO_NUMBER),
    ("sistemas", SISTEMA_TO_NUMBER),
    ("categorias", CATEGORIA_TO_NUMBER),
    ("equipes sistemas", EQUIPE_TO_NUMBER),
    ("equipes infraestrutura", EQUIPE_INFRAESTRUTURA_TO_NUMBER),
    ("equipe geral", EQUIPE_GERAL_TO_NUMBER),
    ("projetos", PROJETO_TO_NUMBER),
    ("linguagens programação", LINGUAGEM_TO_NUMBER_OS_SISTEMAS),
    ("os interna", OS_INTERNA_OS_TO_NUMBER),
    ("status os", STATUS_OS_TO_NUMBER),
    ("criticidade", CRITICIDADE_OS_TO_NUMBER),
    ("prioridade do usuario os", PRIORIDADE_USUARIO_OS_TO_NUMBER),
    ("situação do usuário os", SITUACAO_USUARIO_TO_NUMBER),
    ("usuarios responsaveis sistemas", USUARIOS_SISTEMAS_DOCSTRING),
    ("usuários responsáveis infraestrutura", USUARIOS_INFRAESTRUTURA_DOCSTRING),
)

SYSTEM_INSTRUCTIONS: Prompt = Prompt.from_text(
    langfuse.get_prompt("siga.developer").compile()
).compile(constantes=constantes_md)

if MCP_TRANSPORT == "stdio":
    SYSTEM_INSTRUCTIONS.compile(
        colaboradores=COLABORADORES_PROMPT,
        data=converter_data_siga("hoje"),
        nome_usuario=NOME_USUARIO_ATUAL,
        matricula=MATRICULA_USUARIO_ATUAL,
        preferencias_usuario=getenv("USER_PREFERENCES", "Sem preferências definidas."),
        constantes=constantes_md
    )



import getpass
from os import getenv
from typing import cast

from siga_mcp._types import TransportType
from rsb.coroutines.run_sync import run_sync
import aiohttp
import ujson


async def __buscar_matricula_por_ad(ad: str) -> tuple[str, str]:
    """Busca a matrícula correspondente a um usuário AD.

    Args:
        ad (str): Nome do usuário AD no formato DOMINIO\\usuario

    Returns:
        int: Matrícula do usuário
    """
    api_key = getenv("AVA_API_KEY")
    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/contasAd/buscarUsuarioAdSigaIA/",
            json={"apiKey": api_key, "descricao": ad},
        ) as response:
            json = await response.json(content_type=None)

            usuario: str = json["result"][0]["USUARIO"]
            nome: str = json["result"][0]["NOME"]
            return usuario, nome


async def __buscar_gestores_dtd() -> list[str]:
    return ["8372", "16500", "24142", "25962"]


# Mapeamento de meses em português
MESES_PT = {
    "janeiro": 1,
    "jan": 1,
    "fevereiro": 2,
    "fev": 2,
    "março": 3,
    "mar": 3,
    "abril": 4,
    "abr": 4,
    "maio": 5,
    "mai": 5,
    "junho": 6,
    "jun": 6,
    "julho": 7,
    "jul": 7,
    "agosto": 8,
    "ago": 8,
    "aug": 8,
    "setembro": 9,
    "set": 9,
    "sep": 9,
    "outubro": 10,
    "out": 10,
    "oct": 10,
    "novembro": 11,
    "nov": 11,
    "dezembro": 12,
    "dez": 12,
    "dec": 12,
}

# Mapeamento de dias da semana em português
DIAS_SEMANA_PT = {
    "segunda": 0,
    "segunda-feira": 0,
    "seg": 0,
    "terça": 1,
    "terça-feira": 1,
    "ter": 1,
    "terca": 1,
    "terca-feira": 1,
    "quarta": 2,
    "quarta-feira": 2,
    "qua": 2,
    "quinta": 3,
    "quinta-feira": 3,
    "qui": 3,
    "sexta": 4,
    "sexta-feira": 4,
    "sex": 4,
    "sábado": 5,
    "sabado": 5,
    "sab": 5,
    "domingo": 6,
    "dom": 6,
}

# Palavras que indicam tempo futuro/passado
TEMPO_FUTURO = [
    "próximo",
    "proximo",
    "que vem",
    "seguinte",
    "vindouro",
    "daqui a",
    "daqui",
    "vindoura",
    "entrante",
]

TEMPO_PASSADO = [
    "passado",
    "anterior",
    "ultimo",
    "último",
    "atrás",
    "atras",
    "ha",
    "há",
    "retrasado",
    "retrasada",
]

# Números por extenso
NUMEROS_EXTENSO = {
    "um": 1,
    "uma": 1,
    "dois": 2,
    "duas": 2,
    "três": 3,
    "tres": 3,
    "quatro": 4,
    "cinco": 5,
    "seis": 6,
    "sete": 7,
    "oito": 8,
    "nove": 9,
    "dez": 10,
    "onze": 11,
    "doze": 12,
    "treze": 13,
    "catorze": 14,
    "quatorze": 14,
    "quinze": 15,
    "dezesseis": 16,
    "dezessete": 17,
    "dezoito": 18,
    "dezenove": 19,
    "vinte": 20,
    "trinta": 30,
}

# Expressões de período do dia
PERIODOS_DIA = {
    "manhã": 8,
    "manha": 8,
    "de manhã": 8,
    "de manha": 8,
    "tarde": 14,
    "de tarde": 14,
    "à tarde": 14,
    "a tarde": 14,
    "noite": 20,
    "de noite": 20,
    "à noite": 20,
    "a noite": 20,
    "madrugada": 2,
    "de madrugada": 2,
}

AD = getpass.getuser()

MCP_TRANSPORT: TransportType = cast(TransportType, getenv("MCP_TRANSPORT")) or "stdio"

# Avoid network calls when running under tests/CI or when explicitly disabled.
_DISABLE_NETWORK = getenv("SIGA_DISABLE_NETWORK", "").lower() in {"1", "true", "yes"}
_PYTEST_RUNNING = getenv("PYTEST_CURRENT_TEST") is not None


def _safe_buscar_matricula() -> tuple[str, str]:
    # Allow overriding via env for deterministic tests
    override = getenv("SIGA_TEST_MATRICULA")
    if override:
        return override, ""
    if MCP_TRANSPORT == "http" or _DISABLE_NETWORK or _PYTEST_RUNNING:
        return "", ""
    try:
        return run_sync(__buscar_matricula_por_ad(AD))
    except Exception:
        # Fall back to empty if lookup fails
        return "", ""


MATRICULA_USUARIO_ATUAL, NOME_USUARIO_ATUAL = _safe_buscar_matricula()


def _safe_buscar_matriculas_liberadas() -> list[str]:
    if _DISABLE_NETWORK or _PYTEST_RUNNING:
        # Minimal default; can be overridden in tests
        return []
    try:
        return run_sync(__buscar_gestores_dtd())
    except Exception:
        return []


MATRICULAS_LIBERADAS = _safe_buscar_matriculas_liberadas()


TEAMS_CLIENT_ID = ""

TEAMS_CLIENT_SECRET = ""

TEAMS_TENANT_ID = ""

PACKAGE_NAME = "siga-mcp"

DEFAULT_PORT = 8000



from textwrap import dedent


def docs() -> str:
    return dedent("""\
        Edita uma Ordem de Serviço existente no sistema SIGA para a área de Infraestrutura.

        **INSTRUÇÃO PARA O AGENTE IA:**
        ANTES de executar esta função de edição:
        1. SEMPRE chame primeiro `buscar_informacoes_os(codigo_os)`
        2. MOSTRE ao usuário os dados atuais vs. dados que serão alterados em formato claro e comparativo
        3. PEÇA confirmação explícita do usuário: "Confirma as alterações? (sim/não)"
        4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
        5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

        Esta função edita uma OS (Ordem de Serviço) existente para **ÁREA INFRAESTRUTURA** (área=2),
        atualizando informações como datas, descrição, tipo, origem, categoria, equipe, projeto e responsável.
        Realiza validação de todos os campos obrigatórios e conversão automática de datas.

        **ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:**
        Quando o usuário solicitar "editar OS" sem especificar área:
        1. **Perguntar qual área**: Sistemas (1) ou Infraestrutura (2)
        2. **Se escolher Infraestrutura**: Usar esta função
        3. **Se escolher Sistemas**: Direcionar para `editar_os_sistemas`

        Args:
            codigo_os: Código/número da OS (Ordem de Serviço) que será editada
            data_solicitacao: Data e hora da solicitação da OS (Ordem de Serviço).
                Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
            assunto: Descrição resumida (título) da OS
            descricao: Descrição detalhada da OS a ser realizada
            matSolicitante: Matrícula do usuário que está solicitando a OS.
            Deve ser "CURRENT_USER" ou um número válido maior que zero. Não aceita valores vazios, "0" ou não-numéricos.
            responsavel: Matrícula do usuário responsável pela OS.
            responsavel_atual: Matrícula do usuário responsável atual pela OS.
            criada_por: Matrícula do usuário que criou a OS.
            prioridade: Código da Solicitação prioritária da OS
            tempo_previsto: Cálculo do tempo previsto para a conclusão da OS
            data_inicio_previsto: Data e hora previsto para iniciar a OS (Ordem de Serviço).
                Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
            data_limite: Data e hora limite para realização da OS (Ordem de Serviço).
                Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
            sprint: Descrição dos sprints
            os_predecessora: Código da OS predecessora da OS em andamento
            chamado_fornecedor: Código do chamado do fornecedor
            rotinas: Descrição da rotina
            plaqueta: Plaqueta relacionado ao equipamento que está sendo atendido.
            os_principal: Código da OS principal
            classificacao: Classificação da OS
            nova: Código para verificar se a OS é nova ou transferência de responsável atual
            data_previsao_entrega: Data e hora da previsão de entrega da OS (Ordem de Serviço).
                Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
            modulo: Módulo da OS
            tempo_restante: Tempo restante para concluir a OS
            ramal: Número do Ramal
            data_envio_email_conclusao: Data e hora para o envio do email de conclusão da OS (Ordem de Serviço).
                Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
            tipo_transacao: código do tipo da transação
            acao: código da ação
            planejamento: Descrição do Planejamento da OS
            grupo: Grupo da OS
            tipo: Tipo da OS, deve ser um dos valores válidos:
            categoria: Categoria relacionado a OS, deve ser um dos valores válidos:
            equipe: Equipe responsável pela OS.
            projeto: Projeto relacionado à OS.
            status: Status da OS, deve ser um dos valores válidos.
            os_interna: Indica se a OS é interna, deve ser um dos valores válidos.
            criticidade: Criticidade da OS, deve ser um dos valores válidos.
            prioridade_usuario: Prioridade definida pelo usuário, deve ser um dos valores válidos.
            origem: Origem da solicitação da OS, deve ser um dos valores válidos.

        Notes:
            - **ATENÇÃO**: Esta operação modifica permanentemente os dados da OS
            - **DIFERENCIAÇÃO DE FUNÇÃO**: Esta função edita OS existente para área infraestrutura (área=2). Para criar nova, use inserir_os_infraestrutura. Para buscar informações de uma OS, use buscar_informacoes_os
            - **VALIDAÇÕES OBRIGATÓRIAS**: Campos matSolicitante e criada_por devem ser "CURRENT_USER" ou números válidos maiores que zero (não aceita valores vazios, "0" ou não-numéricos)
            - A função realiza validação case-insensitive de todos os campos (tipo, origem, categoria, equipe, projeto) 
            - As datas são automaticamente convertidas usando converter_data_siga com manter_horas=True 
            - Utiliza as constantes: TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA, CATEGORIA_TO_NUMBER, EQUIPE_INFRAESTRUTURA_TO_NUMBER, PROJETO_TO_NUMBER, STATUS_OS_TO_NUMBER, OS_INTERNA_OS_TO_NUMBER, ORIGEM_OS_TO_NUMBER, PRIORIDADE_USUARIO_OS_TO_NUMBER, CRITICIDADE_OS_TO_NUMBER para mapear valores para códigos numéricos 
            - Todos os parâmetros enviados são incluídos como atributos no XML de resposta 
            - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY 
            - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML 
            - O resultado da API (1 = sucesso, outros valores = erro) é interpretado automaticamente 
            - Campos opcionais como nomeSolicitante, centroCusto, etc. são enviados vazios por padrão 
            - Esta função atualiza OS (Ordens de Serviços) existentes
        """)



from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime, time, timedelta
import re
import xml.etree.ElementTree as ET
from typing import Literal, Sequence
from enum import Enum


# -------------------- Value Objects --------------------


@dataclass(frozen=True, slots=True)
class HoraMinuto:
    """Representa um horário do dia com precisão de minutos (HH:MM).

    Exemplos
    --------
    >>> h = HoraMinuto.from_string("7:05")
    >>> str(h)
    '07:05'
    >>> h.to_time()
    datetime.time(7, 5)
    >>> HoraMinuto(23, 59).minutes_since_midnight()
    1439
    """

    hour: int
    minute: int

    _REGEX = re.compile(r"^(\d{1,2}):(\d{2})$")

    def __post_init__(self):
        if not (0 <= self.hour < 24 and 0 <= self.minute < 60):
            raise ValueError("HoraMinuto inválido: fora do intervalo 00:00..23:59")

    @staticmethod
    def from_string(value: str) -> HoraMinuto:
        """Cria um horário a partir de uma string 'HH:MM'.

        Exemplos
        --------
        >>> h = HoraMinuto.from_string('7:05')
        >>> (h.hour, h.minute)
        (7, 5)
        >>> str(h)
        '07:05'
        """
        value = value.strip()
        m = HoraMinuto._REGEX.match(value)
        if not m:
            raise ValueError(f"Formato inválido de HH:MM: {value!r}")
        hh, mm = int(m.group(1)), int(m.group(2))
        return HoraMinuto(hh, mm)

    def __str__(self) -> str:
        """Representação normalizada no formato HH:MM.

        >>> str(HoraMinuto(7, 5))
        '07:05'
        """
        return f"{self.hour:02d}:{self.minute:02d}"

    def to_time(self) -> time:
        """Converte para ``datetime.time``.

        >>> HoraMinuto(7, 5).to_time()
        datetime.time(7, 5)
        """
        return time(self.hour, self.minute)

    def minutes_since_midnight(self) -> int:
        """Minutos decorridos desde 00:00.

        >>> HoraMinuto(1, 30).minutes_since_midnight()
        90
        """
        return self.hour * 60 + self.minute

    def __lt__(self, other: "HoraMinuto") -> bool:  # para ordenação/comparação simples
        """True se este horário for anterior ao outro.

        >>> HoraMinuto(8, 0) < HoraMinuto(9, 0)
        True
        """
        return self.minutes_since_midnight() < other.minutes_since_midnight()

    def __le__(self, other: "HoraMinuto") -> bool:
        """True se este horário for anterior ou igual ao outro.

        >>> HoraMinuto(8, 0) <= HoraMinuto(8, 0)
        True
        """
        return self.minutes_since_midnight() <= other.minutes_since_midnight()

    def __eq__(self, other: object) -> bool:
        """Compara igualdade por hora e minuto.

        >>> HoraMinuto(8, 0) == HoraMinuto(8, 0)
        True
        >>> HoraMinuto(8, 1) == HoraMinuto(8, 0)
        False
        """
        if not isinstance(other, HoraMinuto):
            return False
        return (self.hour, self.minute) == (other.hour, other.minute)

    def __ge__(self, other: "HoraMinuto") -> bool:
        """True se este horário for posterior ou igual ao outro.

        >>> HoraMinuto(9, 0) >= HoraMinuto(8, 59)
        True
        """
        return self.minutes_since_midnight() >= other.minutes_since_midnight()

    def __gt__(self, other: "HoraMinuto") -> bool:
        """True se este horário for posterior ao outro.

        >>> HoraMinuto(9, 0) > HoraMinuto(8, 59)
        True
        """
        return self.minutes_since_midnight() > other.minutes_since_midnight()

    def shifted(self, minutes: int, *, clamp: bool = True) -> HoraMinuto:
        """Retorna um novo horário deslocado em minutos.

        Se clamp=True (padrão), limita no intervalo 00:00..23:59.
        Se clamp=False, faz wrap-around em 24h.

        Exemplos
        --------
        >>> str(HoraMinuto(8, 0).shifted(90))
        '09:30'
        >>> str(HoraMinuto(0, 10).shifted(-30))  # clamp evita negativos
        '00:00'
        >>> str(HoraMinuto(23, 50).shifted(20, clamp=False))  # wrap 24h
        '00:10'
        """
        total = self.minutes_since_midnight() + int(minutes)
        if clamp:
            total = max(0, min(23 * 60 + 59, total))
        else:
            total %= 24 * 60
        h, m = divmod(total, 60)
        return HoraMinuto(h, m)

    def diff(self, other: HoraMinuto) -> "DuracaoMinutos":
        """Diferença absoluta entre dois horários (em minutos).

        >>> HoraMinuto(9, 0).diff(HoraMinuto(8, 30)).minutes
        30
        """
        return DuracaoMinutos(
            abs(self.minutes_since_midnight() - other.minutes_since_midnight())
        )


@dataclass(frozen=True, slots=True)
class DuracaoMinutos:
    """Duração em minutos com utilitários para conversão.

    >>> DuracaoMinutos(150).to_hhmm()
    '02:30'
    >>> DuracaoMinutos.from_hhmm('08:00').minutes
    480
    >>> (DuracaoMinutos(120) + DuracaoMinutos(45)).minutes
    165
    """

    minutes: int

    def __post_init__(self):
        if self.minutes < 0:
            object.__setattr__(self, "minutes", 0)

    @staticmethod
    def from_hhmm(value: str) -> DuracaoMinutos:
        """Cria uma duração a partir de 'HH:MM'.

        >>> DuracaoMinutos.from_hhmm('01:30').minutes
        90
        >>> str(DuracaoMinutos.from_hhmm('00:45'))
        '00:45'
        """
        h = HoraMinuto.from_string(value)
        return DuracaoMinutos(h.hour * 60 + h.minute)

    def to_hhmm(self) -> str:
        """Retorna representação HH:MM da duração.

        >>> DuracaoMinutos(90).to_hhmm()
        '01:30'
        """
        h, m = divmod(self.minutes, 60)
        return f"{h:02d}:{m:02d}"

    def __add__(self, other: DuracaoMinutos) -> DuracaoMinutos:
        """Soma duas durações.

        >>> (DuracaoMinutos(30) + DuracaoMinutos(45)).minutes
        75
        """
        return DuracaoMinutos(self.minutes + other.minutes)

    def __sub__(self, other: DuracaoMinutos) -> DuracaoMinutos:
        """Subtrai durações com piso em 0 minutos.

        >>> (DuracaoMinutos(90) - DuracaoMinutos(15)).minutes
        75
        >>> (DuracaoMinutos(10) - DuracaoMinutos(20)).minutes
        0
        """
        return DuracaoMinutos(max(0, self.minutes - other.minutes))

    # Comparações ricas para facilitar testes e regras
    def __lt__(self, other: "DuracaoMinutos") -> bool:
        """True se esta duração for menor que a outra.

        >>> DuracaoMinutos(59) < DuracaoMinutos(60)
        True
        """
        return self.minutes < other.minutes

    def __le__(self, other: "DuracaoMinutos") -> bool:
        """True se esta duração for menor ou igual à outra.

        >>> DuracaoMinutos(60) <= DuracaoMinutos(60)
        True
        """
        return self.minutes <= other.minutes

    def __eq__(self, other: object) -> bool:
        """Compara igualdade pela quantidade de minutos.

        >>> DuracaoMinutos(15) == DuracaoMinutos(15)
        True
        >>> DuracaoMinutos(15) == DuracaoMinutos(16)
        False
        """
        if not isinstance(other, DuracaoMinutos):
            return False
        return self.minutes == other.minutes

    def to_timedelta(self):
        """Converte para ``datetime.timedelta``.

        >>> DuracaoMinutos(90).to_timedelta()
        datetime.timedelta(seconds=5400)
        """
        from datetime import timedelta

        return timedelta(minutes=self.minutes)

    @staticmethod
    def from_timedelta(td: timedelta) -> "DuracaoMinutos":
        """Cria uma duração a partir de um ``timedelta``.

        >>> DuracaoMinutos.from_timedelta(timedelta(hours=2)).minutes
        120
        """
        return DuracaoMinutos(int(td.total_seconds() // 60))

    @staticmethod
    def sum(values: Sequence["DuracaoMinutos"]) -> "DuracaoMinutos":
        """Soma uma sequência de durações.

        >>> DuracaoMinutos.sum([DuracaoMinutos(10), DuracaoMinutos(20)]).minutes
        30
        """
        return DuracaoMinutos(sum(v.minutes for v in values))

    def scaled(self, factor: float) -> "DuracaoMinutos":
        """Retorna uma nova duração escalonada pelo fator.

        >>> DuracaoMinutos(60).scaled(1.5).to_hhmm()
        '01:30'
        """
        return DuracaoMinutos(int(round(self.minutes * factor)))


# -------------------- Small Value Objects --------------------


@dataclass(frozen=True, slots=True)
class Matricula:
    value: str

    def __str__(self) -> str:
        """Retorna a string da matrícula.

        >>> str(Matricula('25962'))
        '25962'
        """
        return self.value


@dataclass(frozen=True, slots=True)
class Percentual:
    value: float  # 0..100

    def __post_init__(self):
        # clamp para segurança
        v = 0.0 if self.value < 0 else 100.0 if self.value > 100 else self.value
        object.__setattr__(self, "value", round(v, 2))

    @staticmethod
    def from_string(s: str) -> Percentual:
        """Cria a partir de strings como '85,5%' ou '85.5'.

        >>> float(Percentual.from_string('85,5%'))
        85.5
        >>> str(Percentual.from_string('200'))  # clamp em 100
        '100.00%'
        """
        s = (s or "").strip().replace("%", "").strip()
        s = s.replace(".", "").replace(",", ".") if "," in s else s
        try:
            return Percentual(float(s))
        except Exception:
            return Percentual(0.0)

    def __float__(self) -> float:
        """Valor numérico entre 0 e 100.

        >>> float(Percentual(12.345))
        12.35
        """
        return self.value

    def __str__(self) -> str:
        """Formata com duas casas e símbolo %.

        >>> str(Percentual(12.3))
        '12.30%'
        """
        return f"{self.value:.2f}%"


class Sistema(Enum):
    SIGA = "SIGA"


@dataclass(frozen=True, slots=True)
class TotalCount:
    value: int

    def __post_init__(self):
        v = self.value if self.value >= 0 else 0
        object.__setattr__(self, "value", v)

    def __int__(self) -> int:
        """Inteiro não negativo.

        >>> int(TotalCount(-5))
        0
        """
        return self.value


@dataclass(frozen=True, slots=True)
class MeioPeriodo:
    """Um período de trabalho (entrada -> saída).

    >>> m = MeioPeriodo(HoraMinuto.from_string('07:50'), HoraMinuto.from_string('12:04'))
    >>> m.duracao.to_hhmm()
    '04:14'
    >>> str(m)
    '07:50-12:04'
    """

    entrada: HoraMinuto
    saida: HoraMinuto

    def __post_init__(self):
        if self.saida < self.entrada:
            raise ValueError("Saída não pode ser antes da entrada")

    @property
    def duracao(self) -> DuracaoMinutos:
        """Duração entre entrada e saída.

        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,30)).duracao.to_hhmm()
        '01:30'
        """
        a = self.entrada.minutes_since_midnight()
        b = self.saida.minutes_since_midnight()
        return DuracaoMinutos(b - a)

    def __str__(self) -> str:
        """Formata como 'HH:MM-HH:MM'.

        >>> str(MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)))
        '08:00-09:00'
        """
        return f"{self.entrada}-{self.saida}"

    def overlaps(self, other: "MeioPeriodo") -> bool:
        """Retorna True se os períodos se sobrepõem.

        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).overlaps(MeioPeriodo(HoraMinuto(8,30), HoraMinuto(9,30)))
        True
        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).overlaps(MeioPeriodo(HoraMinuto(9,0), HoraMinuto(10,0)))
        False
        """
        return not (self.saida <= other.entrada or other.saida <= self.entrada)

    def contains_time(self, t: HoraMinuto) -> bool:
        """True se t estiver dentro do período (fechado nos extremos).

        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).contains_time(HoraMinuto(8,0))
        True
        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).contains_time(HoraMinuto(9,1))
        False
        """
        return self.entrada <= t <= self.saida

    def gap_to(self, next_period: "MeioPeriodo") -> DuracaoMinutos:
        """Gap entre este período e o próximo (0 se sobrepõe ou encosta).

        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).gap_to(MeioPeriodo(HoraMinuto(10,0), HoraMinuto(11,0))).to_hhmm()
        '01:00'
        >>> MeioPeriodo(HoraMinuto(8,0), HoraMinuto(9,0)).gap_to(MeioPeriodo(HoraMinuto(9,0), HoraMinuto(10,0))).to_hhmm()
        '00:00'
        """
        if next_period.entrada <= self.saida:
            return DuracaoMinutos(0)
        return next_period.entrada.diff(self.saida)


@dataclass(frozen=True, slots=True)
class PeriodoCompleto:
    periodos: list[MeioPeriodo]

    def total_duracao(self) -> DuracaoMinutos:
        """Soma das durações dos períodos.

        >>> pc = PeriodoCompleto([MeioPeriodo(HoraMinuto(8,0), HoraMinuto(12,0)), MeioPeriodo(HoraMinuto(13,0), HoraMinuto(17,0))])
        >>> pc.total_duracao().to_hhmm()
        '08:00'
        """
        total = 0
        for p in self.periodos:
            total += p.duracao.minutes
        return DuracaoMinutos(total)

    def tem_dois_periodos_ou_mais(self) -> bool:
        """Verifica se há dois ou mais períodos.

        >>> pc = PeriodoCompleto([MeioPeriodo(HoraMinuto(8,0), HoraMinuto(12,0)), MeioPeriodo(HoraMinuto(13,0), HoraMinuto(17,0))])
        >>> pc.tem_dois_periodos_ou_mais()
        True
        >>> pc = PeriodoCompleto([MeioPeriodo(HoraMinuto(8,0), HoraMinuto(12,0))])
        >>> pc.tem_dois_periodos_ou_mais()
        False
        """
        return len(self.periodos) >= 2

    def eh_consistente_com_carga_horaria(
        self, carga_horaria: DuracaoMinutos, tolerancia_min: int = 0
    ) -> bool:
        """Compara total de períodos com carga_horaria, com tolerância.

        >>> pc = PeriodoCompleto([MeioPeriodo(HoraMinuto(8,0), HoraMinuto(12,0)), MeioPeriodo(HoraMinuto(13,0), HoraMinuto(17,0))])
        >>> pc.eh_consistente_com_carga_horaria(DuracaoMinutos(480))
        True
        >>> pc.eh_consistente_com_carga_horaria(DuracaoMinutos(450), tolerancia_min=20)
        True
        >>> pc.eh_consistente_com_carga_horaria(DuracaoMinutos(450), tolerancia_min=10)
        False
        """
        diff = abs(self.total_duracao().minutes - carga_horaria.minutes)
        return diff <= max(0, tolerancia_min)

    def alterar_periodos(self, novos_periodos: list[MeioPeriodo]) -> PeriodoCompleto:
        """Retorna novo PeriodoCompleto com os períodos alterados.

        >>> pc = PeriodoCompleto([MeioPeriodo(HoraMinuto(8,0), HoraMinuto(12,0))])
        >>> pc2 = pc.alterar_periodos([MeioPeriodo(HoraMinuto(13,0), HoraMinuto(17,0))])
        >>> len(pc.periodos)
        1
        >>> len(pc2.periodos)
        1
        >>> str(pc2.periodos[0])
        '13:00-17:00'
        """
        return PeriodoCompleto(novos_periodos)


# -------------------- Entities/Aggregates --------------------


@dataclass(frozen=True, slots=True)
class Analista:
    """Entidade analista.

    >>> Analista(matricula='25962', nome='ARTHUR')
    Analista(matricula='25962', nome='ARTHUR')
    """

    matricula: Matricula
    nome: str


@dataclass(slots=True)
class DiaDeTrabalho:
    """Agregado que modela um dia de trabalho.

    Mantém batidas (pontos registrados), períodos derivados, carga horária e tempo registrado.
    """

    data: date
    carga_horaria: DuracaoMinutos
    tempo_registrado: DuracaoMinutos
    batidas: list[HoraMinuto]

    @staticmethod
    def parse_batidas(ponto_registrado: str) -> list[HoraMinuto]:
        """Converte string CSV de batidas para lista de HoraMinuto.

        >>> DiaDeTrabalho.parse_batidas('08:00, 12:00, 13:00, 18:00')
        [HoraMinuto(hour=8, minute=0), HoraMinuto(hour=12, minute=0), HoraMinuto(hour=13, minute=0), HoraMinuto(hour=18, minute=0)]
        >>> DiaDeTrabalho.parse_batidas('inválido, 08:00')  # ignora inválidos
        [HoraMinuto(hour=8, minute=0)]
        """
        tokens = [t.strip() for t in (ponto_registrado or "").split(",") if t.strip()]
        batidas: list[HoraMinuto] = []
        for t in tokens:
            try:
                batidas.append(HoraMinuto.from_string(t))
            except Exception:
                # Ignora tokens inválidos
                continue
        return batidas

    @staticmethod
    def _periodos_from_batidas(batidas: Sequence[HoraMinuto]) -> list[MeioPeriodo]:
        """Agrupa batidas em pares (entrada, saída) válidos.

        >>> b = [HoraMinuto(8,0), HoraMinuto(12,0), HoraMinuto(13,0), HoraMinuto(17,0)]
        >>> [str(p) for p in DiaDeTrabalho._periodos_from_batidas(b)]
        ['08:00-12:00', '13:00-17:00']
        """
        periodos: list[MeioPeriodo] = []
        for i in range(0, len(batidas) - 1, 2):
            e, s = batidas[i], batidas[i + 1]
            if s < e:
                # ignora par inconsistente
                continue
            periodos.append(MeioPeriodo(e, s))
        return periodos

    @property
    def periodos(self) -> list[MeioPeriodo]:
        """Períodos derivados das batidas."""
        return self._periodos_from_batidas(self.batidas)

    @property
    def total_minutos_periodos(self) -> DuracaoMinutos:
        """Soma das durações dos períodos.

        >>> d = DiaDeTrabalho(date.today(), DuracaoMinutos(480), DuracaoMinutos(0), [HoraMinuto(8,0), HoraMinuto(12,0)])
        >>> d.total_minutos_periodos.to_hhmm()
        '04:00'
        """
        total = 0
        for p in self.periodos:
            total += p.duracao.minutes
        return DuracaoMinutos(total)

    def percentual_registrado(self) -> float | None:
        """Percentual do tempo oficialmente registrado em relação à carga.

        >>> d = DiaDeTrabalho(date.today(), DuracaoMinutos(480), DuracaoMinutos(240), [])
        >>> d.percentual_registrado()
        50.0
        """
        if self.carga_horaria.minutes <= 0:
            return None
        return round(
            (self.tempo_registrado.minutes / self.carga_horaria.minutes) * 100, 2
        )

    def faltam_batidas(self) -> bool:
        """True se número de batidas for ímpar (faltando saída)."""
        return len(self.batidas) % 2 == 1

    def completar_batidas(
        self,
        horarios_esperados: Sequence[MeioPeriodo],
        *,
        mode: Literal["append", "add_missing"] = "append",
    ) -> None:
        """Completa batidas faltantes segundo o padrão esperado.

        Modos suportados
        ----------------
        - "append" (padrão): assume que ``horarios_esperados`` descreve o dia completo
          na ordem cronológica e adiciona as batidas que faltam a partir da
          quantidade já existente. Não reordena nem substitui.
        - "add_missing": para cada batida esperada, adiciona ao final se ainda não
          existir na lista atual (útil quando você passa apenas um meio-período,
          por exemplo só a tarde).

        Exemplos
        --------
        >>> d = DiaDeTrabalho(
        ...     data=date(2025, 1, 1),
        ...     carga_horaria=DuracaoMinutos.from_hhmm('08:00'),
        ...     tempo_registrado=DuracaoMinutos.from_hhmm('02:30'),
        ...     batidas=[HoraMinuto.from_string('07:50'), HoraMinuto.from_string('12:04')]
        ... )
        >>> d.completar_batidas([MeioPeriodo(HoraMinuto.from_string('13:58'), HoraMinuto.from_string('18:16'))])
        >>> [str(b) for b in d.batidas]
        ['07:50', '12:04', '13:58', '18:16']

        Passando apenas o período da tarde
        ----------------------------------
        >>> d2 = DiaDeTrabalho(
        ...     data=date(2025, 1, 1),
        ...     carga_horaria=DuracaoMinutos.from_hhmm('08:00'),
        ...     tempo_registrado=DuracaoMinutos.from_hhmm('02:00'),
        ...     batidas=[HoraMinuto(8, 0), HoraMinuto(10, 0)]
        ... )
        >>> d2.completar_batidas([MeioPeriodo(HoraMinuto(13, 0), HoraMinuto(17, 0))], mode='add_missing')
        >>> [str(b) for b in d2.batidas]
        ['08:00', '10:00', '13:00', '17:00']
        """
        esperadas: list[HoraMinuto] = []
        for mp in horarios_esperados:
            esperadas.append(mp.entrada)
            esperadas.append(mp.saida)

        if not esperadas:
            return

        if mode not in {"append", "add_missing"}:
            raise ValueError("mode deve ser 'append' ou 'add_missing'")

        if mode == "append":
            if len(self.batidas) >= len(esperadas):
                return
            faltantes = esperadas[len(self.batidas) :]
            self.batidas.extend(faltantes)
        else:  # add_missing
            existentes = {(b.hour, b.minute) for b in self.batidas}
            for hm in esperadas:
                key = (hm.hour, hm.minute)
                if key not in existentes:
                    self.batidas.append(hm)
                    existentes.add(key)

    # --------- Novos utilitários ---------
    def minutos_trabalhados(self) -> DuracaoMinutos:
        """Minutos trabalhados derivados dos períodos."""
        return self.total_minutos_periodos

    def horas_restantes(self) -> DuracaoMinutos:
        """Horas restantes com base no tempo_registrado oficial."""
        return self.carga_horaria - self.tempo_registrado

    def is_consistente(self, tolerancia_min: int = 0) -> bool:
        """Compara total de períodos com tempo_registrado, com tolerância.

        >>> d = DiaDeTrabalho(date.today(), DuracaoMinutos(480), DuracaoMinutos(240), [HoraMinuto(8,0), HoraMinuto(12,1)])
        >>> d.is_consistente(tolerancia_min=2)
        True
        """
        diff = abs(self.total_minutos_periodos.minutes - self.tempo_registrado.minutes)
        return diff <= max(0, tolerancia_min)

    def recomputar_tempo_registrado(self) -> None:
        """Define tempo_registrado como a soma dos períodos derivados."""
        self.tempo_registrado = self.total_minutos_periodos

    def proxima_batida_esperada(self) -> str:
        """'entrada' ou 'saida' conforme o número atual de batidas."""
        return "entrada" if len(self.batidas) % 2 == 0 else "saida"

    def adicionar_batida(self, hm: HoraMinuto) -> None:
        """Adiciona batida mantendo ordenação e evitando duplicados consecutivos."""
        # Insere mantendo ordenação e evitando duplicados imediatos
        if not self.batidas:
            self.batidas.append(hm)
            return
        if self.batidas[-1] == hm:
            return
        if hm >= self.batidas[-1]:
            self.batidas.append(hm)
        else:
            # insere ordenado
            for i, b in enumerate(self.batidas):
                if hm < b:
                    self.batidas.insert(i, hm)
                    break

    def registrar_periodo(self, mp: MeioPeriodo) -> None:
        """Atalho para inserir as duas batidas do período."""
        self.adicionar_batida(mp.entrada)
        self.adicionar_batida(mp.saida)


@dataclass(slots=True)
class PendenciaLancamentos:
    """Uma pendência (um dia) de lançamentos para um analista."""

    analista: Analista
    dia: DiaDeTrabalho
    percentual: Percentual | None = None

    # --------- Factories (XML) ---------
    @staticmethod
    def _safe_text(node: ET.Element | None) -> str:
        """Extrai texto do nó, retornando '' se None."""
        if node is None or node.text is None:
            return ""
        return node.text.strip()

    @classmethod
    def from_item_node(cls, item: ET.Element) -> PendenciaLancamentos:
        """Cria uma pendência a partir de um nó <pendencias_lançamentos> (item).

        Exemplo mínimo de XML de item
        ----------------------------
        <pendencias_lançamentos>
            <analista>25962</analista>
            <nome_analista>ARTHUR</nome_analista>
            <dt_atendimento>01/01/25</dt_atendimento>
            <carga_horaria>08:00</carga_horaria>
            <tempo_registrado>02:00</tempo_registrado>
            <percentual>25,00%</percentual>
            <ponto_registrado>08:00, 12:00</ponto_registrado>
        </pendencias_lançamentos>
        """

        def gx(tag: str) -> str:
            return cls._safe_text(item.find(tag))

        matricula = Matricula(gx("analista"))
        nome = gx("nome_analista")
        dt_str = gx("dt_atendimento")
        ch = gx("carga_horaria") or "00:00"
        tr = gx("tempo_registrado") or "00:00"
        perc_str = gx("percentual")
        ponto = gx("ponto_registrado")

        try:
            dt = datetime.strptime(dt_str, "%d/%m/%y").date()
        except Exception:
            dt = date.today()

        batidas = DiaDeTrabalho.parse_batidas(ponto)
        dia = DiaDeTrabalho(
            data=dt,
            carga_horaria=DuracaoMinutos.from_hhmm(ch),
            tempo_registrado=DuracaoMinutos.from_hhmm(tr),
            batidas=batidas,
        )
        analista = Analista(matricula=matricula, nome=nome)

        return cls(
            analista=analista,
            dia=dia,
            percentual=Percentual.from_string(perc_str) if perc_str else None,
        )

    # --------- Consultas/derivações ---------
    def minutos_pendentes(self) -> DuracaoMinutos:
        # com base nos campos oficiais informados no item
        return self.dia.carga_horaria - self.dia.tempo_registrado

    def faltam_batidas(self) -> bool:
        return self.dia.faltam_batidas()

    def percentual_registrado_float(self) -> float | None:
        return self.dia.percentual_registrado()


@dataclass(frozen=True, slots=True)
class PendenciasLancamentos:
    """Agregado que representa o retorno completo do XML com várias pendências.

    - matricula: matrícula alvo da consulta (do elemento raiz)
    - total: número de itens pendentes (do elemento raiz)
    - sistema: identificador do sistema (do elemento raiz)
    - itens: lista de pendências individuais (um dia por item)
    """

    matricula: Matricula | None
    total: TotalCount | None
    sistema: Sistema | None
    itens: list[PendenciaLancamentos]

    @classmethod
    def from_xml(cls, xml: str) -> PendenciasLancamentos:
        """Interpreta o XML raiz e popula o agregado completo.

        Inclui metadados do elemento raiz (matricula, total, sistema) e
        uma lista de itens (um por dia pendente).
        """
        root = ET.fromstring(xml)
        meta = dict(root.attrib)
        matricula = Matricula(meta["matricula"]) if "matricula" in meta else None
        total: TotalCount | None
        try:
            total = TotalCount(int(meta["total"])) if "total" in meta else None
        except Exception:
            total = None
        sistema = Sistema(meta["sistema"]) if "sistema" in meta else None

        itens: list[PendenciaLancamentos] = []
        # Cada filho <pendencias_lançamentos> representa um dia
        for child in list(root):
            if child.tag != "pendencias_lançamentos":
                continue
            itens.append(PendenciaLancamentos.from_item_node(child))

        # Caso o XML tenha vindo como item único sem filho, tentar interpretar o root
        if not itens and root.tag == "pendencias_lançamentos":
            itens.append(PendenciaLancamentos.from_item_node(root))

        return cls(matricula=matricula, total=total, sistema=sistema, itens=itens)

    # --------- Operações sobre coleção ---------
    def total_dias(self) -> int:
        """Quantidade de dias pendentes.

        >>> PendenciasLancamentos(None, None, None, []).total_dias()
        0
        """
        return len(self.itens)

    def dias_com_falta_batida(self) -> list[PendenciaLancamentos]:
        """Filtra itens com batidas ímpares."""
        return [p for p in self.itens if p.faltam_batidas()]

    def minutos_pendentes_totais(self) -> DuracaoMinutos:
        """Soma dos minutos pendentes dos itens."""
        return DuracaoMinutos.sum([p.minutos_pendentes() for p in self.itens])

    def completar_batidas_todos(
        self,
        horarios_esperados: Sequence[MeioPeriodo],
        *,
        mode: Literal["append", "add_missing"] = "append",
    ) -> None:
        """Completa batidas faltantes de todos os itens conforme um padrão esperado.

        Comportamento
        -------------
        - Converte a sequência de ``MeioPeriodo`` em uma lista linear de batidas
            esperadas [entrada1, saída1, entrada2, saída2, ...].
                - Para cada item do agregado:
                    - Em mode="append": se o número de batidas atual é menor que o número de
                        batidas esperadas, anexa (append-only) as batidas faltantes ao final,
                        sem reordenar nem substituir batidas já existentes.
                    - Em mode="add_missing": para cada batida esperada, adiciona ao final se
                        ainda não existir na lista do item (não reordena o que já existe).
        - Se um item já tem o mesmo número (ou mais) de batidas que o esperado, nada é feito para esse item.
        - Operação destrutiva: modifica os objetos ``DiaDeTrabalho`` in place.

        Exemplos
        --------
        >>> agg = PendenciasLancamentos(
        ...     matricula=Matricula('25962'),
        ...     total=TotalCount(1),
        ...     sistema=Sistema.SIGA,
        ...     itens=[
        ...         PendenciaLancamentos(
        ...             analista=Analista(matricula=Matricula('25962'), nome='ARTHUR'),
        ...             dia=DiaDeTrabalho(
        ...                 data=date(2025, 1, 1),
        ...                 carga_horaria=DuracaoMinutos.from_hhmm('08:00'),
        ...                 tempo_registrado=DuracaoMinutos.from_hhmm('02:00'),
        ...                 batidas=[HoraMinuto(8, 0), HoraMinuto(10, 0)]
        ...             )
        ...         )
        ...     ]
        ... )
        >>> esperado = [MeioPeriodo(HoraMinuto(13, 0), HoraMinuto(17, 0))]
        >>> agg.completar_batidas_todos(esperado, mode='add_missing')
        >>> [str(b) for b in agg.itens[0].dia.batidas]
        ['08:00', '10:00', '13:00', '17:00']
        """
        for p in self.itens:
            p.dia.completar_batidas(horarios_esperados, mode=mode)
